# Netmode Daemon
The netmode daemon (netmoded), along with netmode scripts, setup the network configuration and state of all Iopsys access points in a network.

Netmoded is responsible for synchronizing the network's wireless and wifi life configuration, as well as flashing the appropriate LEDs when the wifi is being synchronized and reconfigured.

## Overview
- subscribes to ubus events to determine when to trigger synchronization
- offers granular and network wide synchronization through ubus and ubusx
- has proc.d trigger setup on configuration files to spread changes through the network

## Run time
At daemon startup, netmode will start event listeners for ```ubus.object.add``` events, and trigger its ```sync``` methods, propagating the wireless and wifi life configuration to any already connected devices.

When an event is received, its path is parsed to see whether it is an remote netmode object, meaning it is prefixed by its ip address, ```{ "ubus.object.add": {"id":689905158,"path":"192.168.1.248\/netmode"} }```. If a remote netmode object is added, its ```sync``` and ```sync_wifilife``` methods are invoked by the master node.

While synchronization is taking place, netmoded (at the node being reconfigured) will add the interfaces to a configuration list, trigger the wifi LEDs to flash for 30 seconds and register listeners for ```wifi.dfs``` and ```wifi.radio``` events. If these events are recorded bringing an interface back up, the interface will be removed from the list, if the configuration list is empty prior to the 30 second timeout, the LEDS will stop flashing.


## Netmode object
Netmoded publishes an object to ubus, ```netmode``.

```
root@iopsys:~# ubus -v list netmode
'netmode' @491cf899
	"status":{}
	"repeat":{"network":"String","wireless":"Table","connect_bssid":"String"}
	"repeat_wifilife":{"wifilife":"Table"}
	"sync":{}
	"sync_wifilife":{}
```

### Methods

#### Status
The ```status``` method provides information about the current access point state, primarily, whether it is in ```routed_*``` or ```repeater_*``` mode. If it is in repeated mode it will also provide the uplink type.

```
root@iopsys:~# ubus call netmode status
{
	"curr_mode": "repeater_mtk_5g_up_dual_down",
	"uplink_type": "ethernet"
}
```

#### Repeat
```repeat``` should be invoked by the master to a remote access point which the configuration should be synchronized. The arguments ```network``` and ```wireless``` must be provided, representing the uci configuration of the corresponding configuration in a json format. The ```connect_bssid``` is an optional argument, which will, if the remote access point is wirelessly connected, force the bssid of the interface it is trying to connect to. The purpose of the connect_bssid option is to preventing daisy chaining repeaters (i.e. a network topology with multiple repeaters connected to eachother). Any pre-existing custom wireless configuration will be overwritten (albeit interface names, radios and network names will be adjusted, based on the device). This method will only overwrite if any options have actually changed, meaning any already synchronized files will not trigger proc.d triggers unnecessarily.

```repeat_wifilife``` offers similar functionality to that of ```repeat```, but takes a wifi life configuration as an argument and has a less dynamic implementation, meaning it will always overwrite the existing configuration (barring the interface names and other device specific options).


#### Sync
```sync``` and ```sync_wifilife``` will propagate the corresponding configuration files through the network by doing a ubus lookup and invoking the ```repeat``` or ```repeat_wifilife``` of every remote netmode object. These methods are invoked at daemon startup to ensure consistent configuration of the network.



