#ifndef _CONFIG_H_
#define _CONFIG_H_

/**
 * config header - functions to read, write &co the uci config files
 *
 * Copyright (C) 2018-2019 iopsys Software Solutions AB. All rights reserved.
 *
 * Author: Reidar Cederqvist <reidar.cederqvist@gmail.com>
 * Author: Ionut-Alex Oprea <ionutalexoprea@gmail.com>
 * Author: Jakob Olsson <jakob.olsson@iopsys.eu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 **/

enum config_compare_status {
	CONFIG_COMPARE_FAIL = -1,
	CONFIG_COMPARE_EQUAL = 0,
	CONFIG_COMPARE_SECTION_CHANGE = 1,
	CONFIG_COMPARE_OPTION_CHANGE = 2,
	CONFIG_COMPARE_WETIF_CHANGE = 3
};
enum config_apply_status {
	CONFIG_APPLY_SUCCESS,
	CONFIG_APPLY_FAIL
};

const char *json_get_string(struct json_object *object, const char *key);
bool config_get_daisy_chain_enabled(void);
enum config_apply_status config_apply_wifilife(struct json_object *remote_cfg);
int config_get_wifilife(struct blob_buf *buf);
int config_get_wireless(struct blob_buf *buf);
enum config_compare_status config_compare_wireless(
	struct json_object *local_config, struct json_object *remote_config,
	struct json_object *remote_ifaces, const char *network,
	const char *connect_bssid);
enum config_apply_status config_apply_wireless(
	struct json_object *remote_config, const char *connect_bssid,
	enum config_compare_status status);
char *config_get_network_from_ip(const char *ip);

int config_ignore_option(const char *option);
void config_free_ignored_options(void);
char *config_get_ifname_by_band(char *ifname, char *band);
int config_get_radios(struct list_head *radios);
#endif /* _CONFIG_H_*/
